/* Header for module lplot, generated by p2c */
#ifndef LPLOT_H
#define LPLOT_H




#ifndef LOGDEF_H
#include "logdef.h"
#endif

#ifndef PLOT_H
#include <p2c/plot.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWCRT_H
#include <p2c/newcrt.h>
#endif

#ifndef MYLIB_H
#include <p2c/mylib.h>
#endif

/*homeless orphans*/

#ifndef FILEPACK_H
#include <p2c/filepack.h>
#endif

#ifndef NUMEX_H
#include <p2c/numex.h>
#endif

#ifndef PLOT_ROUTINES_H
#include <p2c/plot_routines.h>
#endif

#ifndef CITINFOMOD_H
#include <p2c/citinfomod.h>
#endif


#ifdef LPLOT_G
# define vextern
#else
# define vextern extern
#endif





extern Void Log_lplot_proc PP((log_action *lact));



#undef vextern

#endif /*LPLOT_H*/

/* End. */
