/* Header for module NSolver, generated by p2c */
#ifndef NSOLVER_H
#define NSOLVER_H



#ifndef GLOBALS_H
#include "globals.h"
#endif

#ifndef LOGDEF_H
#include "logdef.h"
#endif


#ifdef NSOLVER_G
# define vextern
#else
# define vextern extern
#endif

/*$If Float$
 $float_hdw on$
 $End$*/
/*$If Debugflag$
 $Debug$
 $End$*/






extern Void Solve PV();


#undef vextern

#endif /*NSOLVER_H*/

/* End. */
