/* Header for module logsimed, generated by p2c */
#ifndef LOGSIMED_H
#define LOGSIMED_H



#ifndef LOGDEF_H
#include "logdef.h"
#endif


#ifndef LOGDIG_H
#include "logdig.h"
#endif

#ifndef NEWKBD_H
#include <p2c/newkbd.h>
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif

#ifndef NEWCRT_H
#include <p2c/newcrt.h>
#endif


#ifdef LOGSIMED_G
# define vextern
#else
# define vextern extern
#endif



extern Void edit_16 PP((uchar **proc, long *proclen, Char *name));
extern Void dump_16 PP((uchar **proc, na_strlist **sl, int indenting));

extern Void read_16 PP((uchar **proc, long *proclen, na_strlist *sl));
extern Char *dasm_16 PP((uchar *proc, long *pc));
extern long indent_16 PP((uchar *proc, long pc));
extern long length_16 PP((uchar *proc, long pc));





extern long assem_16 PP((Char *buf, uchar *proc, long *pc));


#undef vextern

#endif /*LOGSIMED_H*/

/* End. */
