/* Header for module logntk, generated by p2c */
#ifndef LOGNTK_H
#define LOGNTK_H




#ifndef LOGDEF_H
#include "logdef.h"
#endif

#ifndef LOGHIER_H
#include "loghier.h"
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWKBD_H
#include <p2c/newkbd.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif

#ifndef NEWCRT_H
#include <p2c/newcrt.h>
#endif

#ifndef MYLIB_H
#include <p2c/mylib.h>
#endif

#ifndef REGEX_H
#include <p2c/regex.h>
#endif


#ifdef LOGNTK_G
# define vextern
#else
# define vextern extern
#endif






extern Void Log_logntk_proc PP((log_action *act));


#undef vextern

#endif /*LOGNTK_H*/

/* End. */
