/* Header for module Main, generated by p2c */
#ifndef MAIN_H
#define MAIN_H





#ifndef NSOLVER_H
#include "nsolver.h"
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif

#ifndef NEWCRT_H
#include <p2c/newcrt.h>
#endif

#ifndef GLOBALS_H
#include "globals.h"
#endif

#ifndef LOGDEF_H
#include "logdef.h"
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef MYLIB_H
#include <p2c/mylib.h>
#endif


#ifdef MAIN_G
# define vextern
#else
# define vextern extern
#endif

/*$If Float$
$float_hdw on$
$End$*/
/*$If Debugflag$
$Debug$
$End$*/


extern Void Analog_Cleanup PV();
extern Void Analog_GlobalCommand PP((boolean *Found, boolean *Accepted));
extern Void Analog_InitVar PV();
extern Void Analog_Newkind PP((log_krec *newkindptr));
extern Void Analog_Probemode PV();
extern Void Analog_Refresh PV();
extern Void Analog_Reset PV();
extern Void Analog_Setup PV();
extern Void Analog_Simulate PV();
extern Void Analog_Status PV();
extern Void Analog_Systeminit PV();
extern Void Analog_TechnologyCnf PV();

extern Void Analog_Tstep PV();


#undef vextern

#endif /*MAIN_H*/

/* End. */
