#include <p2c/p2c.h>
#include "logstuff.h"

/* Do not modify this file.  It is created automatically by "munch". */

extern Void Log_0_proc();
extern Void Log_16_proc();
extern Void Log_17_proc();
extern Void Log_1_proc();
extern Void Log_2_proc();
extern Void Log_32_proc();
extern Void Log_33_proc();
extern Void Log_7_proc();

struct ext_proc ext_proc_table[] = {
  "Log_0_proc", Log_0_proc,
  "Log_16_proc", Log_16_proc,
  "Log_17_proc", Log_17_proc,
  "Log_1_proc", Log_1_proc,
  "Log_2_proc", Log_2_proc,
  "Log_32_proc", Log_32_proc,
  "Log_33_proc", Log_33_proc,
  "Log_7_proc", Log_7_proc,
  NULL, NULL
};

