/* Header for module loghier, generated by p2c */
#ifndef LOGHIER_H
#define LOGHIER_H




#ifndef LOGDEF_H
#include "logdef.h"
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef MYLIB_H
#include <p2c/mylib.h>
#endif


#ifdef LOGHIER_G
# define vextern
#else
# define vextern extern
#endif




typedef short log_shintarray[log_million];


extern Void hier_init PP((log_action *act));
extern boolean commandlabel PP((log_lrec *lp, Char *cmd, Char *args));
extern log_brec *labelbox PP((log_lrec *lp, int pg));
extern boolean pointinbox PP((log_brec *bp, int x, int y));
extern boolean gateinbox PP((log_brec *bp, log_grec *g));
extern boolean labelinbox PP((log_brec *bp, log_lrec *l));
extern boolean boxinbox PP((log_brec *bp, log_brec *bp2));
extern boolean boxtouchbox PP((log_brec *bp, log_brec *bp2));
extern boolean isinstgate PP((log_grec *g));
extern boolean isgenericinstgate PP((log_grec *g));
extern Char *gateinstof PP((Char *Result, log_grec *g));
extern Char *gateinstname PP((Char *Result, log_grec *g));
extern na_strlist *defndirectory PP((int pg));
extern Void examinetemplate PP((log_grec *g, log_nrec **pp, long ppsize,
				int isgeneric, short **pnum, long *lastnorth,
				long *lasteast, long *lastsouth,
				long *lastwest));



extern Void Log_7_ginst PP((log_action *act));


#undef vextern

#endif /*LOGHIER_H*/

/* End. */
