/* Header for module logsim, generated by p2c */
#ifndef LOGSIM_H
#define LOGSIM_H



#ifndef LOGDEF_H
#include "logdef.h"
#endif


#ifndef LOGDIG_H
#include "logdig.h"
#endif

#ifndef LOGSIMA_H
#include "logsima.h"
#endif


#ifndef LOGSIMED_H
#include "logsimed.h"
#endif

#ifndef RND_H
#include <p2c/rnd.h>
#endif

#ifndef NEWKBD_H
#include <p2c/newkbd.h>
#endif

#ifndef NEWASM_H
#include <p2c/newasm.h>
#endif

#ifndef NEWCI_H
#include <p2c/newci.h>
#endif

#ifndef NEWCRT_H
#include <p2c/newcrt.h>
#endif

#ifndef MYLIB_H
#include <p2c/mylib.h>
#endif


#ifdef LOGSIM_G
# define vextern
#else
# define vextern extern
#endif




extern Void Log_16_proc PP((log_action *lact));
extern Void Log_16_7seg PP((log_16_action *act));
extern Void Log_16_break PP((log_16_action *act));
extern Void Log_16_clock PP((log_16_action *act));
extern Void Log_16_keypad PP((log_16_action *act));
extern Void Log_16_ledgate PP((log_16_action *act));
extern Void Log_16_ledgate2 PP((log_16_action *act));
extern Void Log_16_pulse PP((log_16_action *act));
extern Void Log_16_scope PP((log_16_action *act));

extern Void Log_16_switch PP((log_16_action *act));
extern Void log_16_conf PP((log_16_action *act, log_nrec *n));



extern Void log_16_noprog PP((log_16_action *act));


#undef vextern

#endif /*LOGSIM_H*/

/* End. */
